//Yugge 2013
package main

import (
	"bytes"
	"code.google.com/p/go.crypto/ssh"
	"encoding/base64"
	"fmt"
	"github.com/nsf/termbox-go"
	"io"
	"io/ioutil"
	"log"
	"os"
	//	"strconv"
	"strings"

//	"time"
)

var (
	server   = ""
	username = ""
	password = clientPassword("")
)

type clientPassword string

func (p clientPassword) Password(user string) (string, error) {
	return string(p), nil
}

func PrintString(x, y int, text string) {
	for i, v := range text {
		termbox.SetCell(x+i, y, v, termbox.ColorWhite, termbox.ColorBlack)
	}
}

func GetUser() string {
	out := ""
	i := 6
	for {
		switch ev := termbox.PollEvent(); ev.Type {
		case termbox.EventKey:
			if ev.Key == termbox.KeyEsc {
				return ""
			} else if ev.Key == termbox.KeyEnter {
				return out
			} else if ev.Key == termbox.KeyBackspace {
				if i != 6 {
					i--
				}
				termbox.SetCell(i, 0, ' ', termbox.ColorWhite, termbox.ColorBlack)
				out = out[:len(out)-1]
				termbox.Flush()
			} else {
				termbox.SetCell(i, 0, ev.Ch, termbox.ColorWhite, termbox.ColorBlack)
				i++
				out += string(ev.Ch)
				termbox.Flush()
			}
		}
	}
}
func GetPass() string {
	out := ""
	i := 10
	for {
		switch ev := termbox.PollEvent(); ev.Type {
		case termbox.EventKey:
			if ev.Key == termbox.KeyEsc {
				return ""
			} else if ev.Key == termbox.KeyEnter {
				return out
			} else if ev.Key == termbox.KeyBackspace {
				if i != 10 {
					i--
				}
				termbox.SetCell(i, 1, ' ', termbox.ColorWhite, termbox.ColorBlack)
				out = out[:len(out)-1]
				termbox.Flush()
			} else {
				termbox.SetCell(i, 1, '*', termbox.ColorWhite, termbox.ColorBlack)
				i++
				out += string(ev.Ch)
				termbox.Flush()
			}
		}
	}
}

type TerminalModes map[uint8]uint32

const (
	VINTR         = 1
	VQUIT         = 2
	VERASE        = 3
	VKILL         = 4
	VEOF          = 5
	VEOL          = 6
	VEOL2         = 7
	VSTART        = 8
	VSTOP         = 9
	VSUSP         = 10
	VDSUSP        = 11
	VREPRINT      = 12
	VWERASE       = 13
	VLNEXT        = 14
	VFLUSH        = 15
	VSWTCH        = 16
	VSTATUS       = 17
	VDISCARD      = 18
	IGNPAR        = 30
	PARMRK        = 31
	INPCK         = 32
	ISTRIP        = 33
	INLCR         = 34
	IGNCR         = 35
	ICRNL         = 36
	IUCLC         = 37
	IXON          = 38
	IXANY         = 39
	IXOFF         = 40
	IMAXBEL       = 41
	ISIG          = 50
	ICANON        = 51
	XCASE         = 52
	ECHO          = 53
	ECHOE         = 54
	ECHOK         = 55
	ECHONL        = 56
	NOFLSH        = 57
	TOSTOP        = 58
	IEXTEN        = 59
	ECHOCTL       = 60
	ECHOKE        = 61
	PENDIN        = 62
	OPOST         = 70
	OLCUC         = 71
	ONLCR         = 72
	OCRNL         = 73
	ONOCR         = 74
	ONLRET        = 75
	CS7           = 90
	CS8           = 91
	PARENB        = 92
	PARODD        = 93
	TTY_OP_ISPEED = 128
	TTY_OP_OSPEED = 129
)

type readWriter struct {
	io.Reader
	io.Writer
}

func NewReadWriter(r io.Reader, w io.Writer) io.ReadWriter {
	return &readWriter{r, w}
}
func main() {

	termbox.Init()
	PrintString(0, 0, "User: ")
	termbox.Flush()
	username = GetUser()
	if username == "" {
		termbox.Close()
		panic("No username..quitting")
	}
	PrintString(0, 1, "Password: ")
	termbox.Flush()
	pass := GetPass()
	if pass == "" {
		termbox.Close()
		panic("No password..quitting")
	}

	termbox.Close()
	password = clientPassword(pass)
	server = "taurus.ludd.ltu.se:22"
	// An SSH client is represented with a slete). Currently only
	// the "password" authentication method is supported.
	//
	// To authenticate with the remote server you must pass at least one
	// implementation of ClientAuth via the Auth field in ClientConfig.

	config := &ssh.ClientConfig{
		User: username,
		Auth: []ssh.ClientAuth{
			// ClientAuthPassword wraps a ClientPassword implementation
			// in a type that implements ClientAuth.
			ssh.ClientAuthPassword(password),
		},
	}
	client, err := ssh.Dial("tcp", server, config)
	if err != nil {
		panic("Failed to dial: " + err.Error())
	}

	// Each ClientConn can support multiple interactive sessions,
	// represented by a Session.
	defer client.Close()
	// Create a session
	session, err := client.NewSession()
	if err != nil {
		log.Fatalf("unable to create session: %s", err)
	}
	defer session.Close()
	fullfilename := ""
	if len(os.Args) > 1 {
		fullfilename = os.Args[1]
	}
	filename_tmp := strings.Split(fullfilename, ".")
	fileext := filename_tmp[len(filename_tmp)-1]
	f, err := ioutil.ReadFile(fullfilename)
	fb64 := []byte(base64.StdEncoding.EncodeToString(f))

	if err != nil {
		panic("Failed to open file: " + err.Error())
	}
	var b bytes.Buffer
	in := bytes.NewBuffer(fb64)
	session.Stdout = &b
	session.Stdin = in
	fmt.Println("Sending to printer...")
	if fileext == "doc" || fileext == "docx" || fileext == "xls" || fileext == "xlsx" {

		if err := session.Run("base64 -d > .print_tmp." + fileext + " && soffice --headless -p .print_tmp." + fileext + "&& rm .print_tmp." + fileext); err != nil {
			panic("Failed to run: " + err.Error())
		}

	} else {

		if err := session.Run("base64 -d | lpr -o fitplot"); err != nil {
			panic("Failed to run: " + err.Error())
		}
	}

}
